# Tekkotsu WorldBuilder
# Shape class definition
#
# Robert Lee (rslee / perihare)
# 2-14-2011

require 'parsevector'
require 'global'

require 'boolean'

class Shape
    attr_accessor :name, :location, :group, :attrs 
    
    def initialize
        @name = "shape-%05d" % $shapecount
        $shapecount = $shapecount + 1
        @attrs ||= {}
        @valid_attrs ||= ['orientation', 'pointat',
            'type', 'kinematics', 'scale', 'material', 'rotation',
            'collision', 'cmodel', 'cmscale', 'cmoffset', 'cmrotation',
            'model', 'moffset', 'mrotation',
            'com', 'visible', 'group']
        @attrs['collision'] = 'true' # shapes collide by default
        
        @group ||= []
        @group.push "all"
        @group.push "real"
    end

    def clone
        s = Shape.new
        @group.each do |groupname|
            s.set_attr('group', groupname)
        end
        @attrs.each do |attr, val|
            s.set_attr(attr, val)
        end
        s.location = @location.clone unless @location.nil?
        return s 
    end

    def set_attr(attr, val)
        if attr.eql? 'group' then
            @group.push val.clone
        elsif attr.eql? 'location' then
            @location = val
        elsif @valid_attrs.include? attr then
            @attrs[attr] = val.clone
        else
            raise "Invalid argument: " + attr.to_s + ", " + val.to_s
        end
    end

    def to_map
        h = Hash.new
        if @location.nil? then
            raise "Missing a location"
        end
        h["Location"] = @location.to_array
        h["Orientation"] = attrs['orientation'].to_array unless attrs['orientation'].nil?
        h["PointAt"] = attrs['pointat'].to_array unless attrs['pointat'].nil?
        h["Kinematics"] = attrs['kinematics'] unless attrs['kinematics'].nil?
        h["CenterOfMass"] = attrs['com'].to_array unless attrs['com'].nil?
        if to_boolean(attrs['collision']) then
            h["CollisionModel"] = (attrs['cmodel'].nil?) ? attrs['type'].capitalize : attrs['cmodel']
            h["CollisionModelOffset"] = attrs['cmoffset'].to_array unless attrs['cmoffset'].nil?
            h["CollisionModelRotation"] = attrs['rotation'].to_array unless attrs['rotation'].nil?
            h["CollisionModelScale"] = attrs['scale'].to_array unless attrs['scale'].nil?
        end
        h["Model"] = attrs['model'] unless attrs['model'].nil?
        h["ModelOffset"] = attrs['moffset'] unless attrs['moffset'].nil?
        h["ModelRotation"] = attrs['mrotation'] unless attrs['mrotation'].nil?
        h["ModelScale"] = attrs['mscale'] unless attrs['mscale'].nil?
        h["Mass"] = attrs['mass'] unless attrs['mass'].nil?
        h["Material"] = attrs['material'] unless attrs['material'].nil?
        h["Visible"] = to_boolean(attrs['visible']) unless attrs['visible'].nil?
        return h
    end

    def flipx
        newlocation = attrs['location']
        newlocation.x = -newlocation.x.to_f
        set_attr('location', newlocation)
        newcmrotation = attrs['cmrotation']
        newcmrotation.z = -newcmrotation.z.to_f unless newcmrotation.nil?
        set_attr('cmrotation', newcmrotation)
    end
    
    def flipy
        newlocation = attrs['location']
        newlocation.y = -newlocation.y.to_f
        set_attr('location', newlocation)
        newcmrotation = attrs['cmrotation']
        newcmrotation.z = -newcmrotation.z.to_f unless newcmrotation.nil?
        set_attr('cmrotation', newcmrotation)
    end
end
